@@ -1,4 +1,4 @@
-# Copyright 2009-2012 by Roger Bivand
+# Copyright 2009-2014 by Roger Bivand
 
 trW <- function(W=NULL, m=30, p=16, type="mult", listw=NULL, momentsSymmetry=TRUE) {
 # returns traces
@@ -116,8 +116,9 @@ impacts <- function(obj, ...)
 
 impacts.stsls <- function(obj, ..., tr=NULL, R=NULL, listw=NULL,
   tol=1e-6, empirical=FALSE, Q=NULL) {
-    if (!is.null(obj$listw_style) && obj$listw_style != "W") 
-        stop("Only row-standardised weights supported")
+    if (is.null(listw) && !is.null(obj$listw_style) && 
+            obj$listw_style != "W")
+            stop("Only row-standardised weights supported")
     rho <- obj$coefficients[1]
     beta <- obj$coefficients[-1]
     icept <- grep("(Intercept)", names(beta))
@@ -147,11 +148,12 @@ impacts.stsls <- function(obj, ..., tr=NULL, R=NULL, listw=NULL,
     res
 }
 
-impacts.gmsar <- function(obj, ..., tr=NULL, R=NULL, listw=NULL,
+impacts.gmsar <- function(obj, ..., n=NULL, tr=NULL, R=NULL, listw=NULL,
   tol=1e-6, empirical=FALSE, Q=NULL) {
-    if (!is.null(obj$listw_style) && obj$listw_style != "W") 
-        stop("Only row-standardised weights supported")
     stopifnot(obj$type == "SARAR") 
+    if (is.null(listw) && !is.null(obj$listw_style) && 
+            obj$listw_style != "W")
+            stop("Only row-standardised weights supported")
     rho <- obj$coefficients[1]
     beta <- obj$coefficients[-1]
     icept <- grep("(Intercept)", names(beta))
@@ -164,7 +166,11 @@ impacts.gmsar <- function(obj, ..., tr=NULL, R=NULL, listw=NULL,
         bnames <- names(beta)
     }
     p <- length(beta)
-    n <- length(obj$residuals)
+# allow n passthrough 140305 Angela Parenti
+    if (is.null(n)) n <- length(obj$residuals)
+    stopifnot(is.integer(n))
+    stopifnot(length(n) == 1)
+    stopifnot(is.finite(n))
     mu <- c(rho, beta)
     Sigma <- obj$secstep_var
     irho <- 1
@@ -281,7 +287,6 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
     .ptime_start <- proc.time()
     if (is.null(listw)) {
 
-
         q <- length(tr)-1L
         g <- rho^(0:q)
         T <- matrix(c(1, tr[-(q+1)]/n), nrow=1)
@@ -356,6 +361,12 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
         }
         attr(res, "method") <- "trace"
     } else {
+# added checks 140304
+        stopifnot(length(listw$neighbours) == n)
+        V <- listw2mat(listw)
+        e <- eigen(V, only.values = TRUE)$values
+        if (is.complex(e)) interval <- 1/(range(Re(e)))
+	else interval <- 1/(range(e))
         SW <- invIrW(listw, rho)
         if (type == "lag") res <- lagImpactsExact(SW, P, n)
         else if (type == "mixed" || type == "sacmixed")
@@ -365,11 +376,9 @@ intImpacts <- function(rho, beta, P, n, mu, Sigma, irho, drop2beta, bnames,
         if (!is.null(R)) {
             samples <- mvrnorm(n=R, mu=mu, Sigma=Sigma, tol=tol,
                 empirical=empirical)
-            if (!is.null(interval)) {
-                check <- ((samples[,irho] > interval[1]) & 
-                    (samples[,irho] < interval[2]))
-                if (any(!check)) samples <- samples[check,]
-            }
+            check <- ((samples[,irho] > interval[1]) & 
+                (samples[,irho] < interval[2]))
+            if (any(!check)) samples <- samples[check,]
             timings[["impacts_samples"]] <- proc.time() - .ptime_start
             .ptime_start <- proc.time()
 # type, iicept, icept, SW, n, listw
@@ -419,8 +428,9 @@ impacts.sarlm <- function(obj, ..., tr=NULL, R=NULL, listw=NULL, useHESS=NULL,
   tol=1e-6, empirical=FALSE, Q=NULL) {
     if (obj$type == "error")
         stop("impact measures not for error models")
-    if (!is.null(obj$listw_style) && obj$listw_style != "W") 
-        stop("Only row-standardised weights supported")
+    if (is.null(listw) && !is.null(obj$listw_style) && 
+            obj$listw_style != "W")
+            stop("Only row-standardised weights supported")
     rho <- obj$rho
     beta <- obj$coefficients
     s2 <- obj$s2
